//
//  ITASDK.h
//  ITASDK
//  版本1.1
//  Created by Tim Wen(wtc@nbicc.com) on 15/1/9.
//  Copyright (C) 2015 The ITACloud Framework SDK Project in Ningbo Institute of Information Technology Application, CAS All rights reserved.
//

#import <Foundation/Foundation.h>

//基础的回调block Parameters:success 成功返回值 error失败返回值

/*NSError:
 NSInteger _code;            错误码
 NSString *_domain;          错误信息
 NSDictionary *_userInfo;    详细内容 一般内容为serialNumber 流水号
 */

//systemMessageCallback
typedef enum
{
    LINK_DISCONNECTED,      //链接断开,请重启服务
    MUTI_LOGIN,             //异地登陆
}SYS_CALLBACK;

@interface ITASDK : NSObject

//用于系统通知的回调
@property (nonatomic, strong) void (^systemMessageCallback)(SYS_CALLBACK);

//UDP回调预留
@property (nonatomic, strong) void (^udpMessageCallback)(NSDictionary*);

//用于平台主动推送的信息
@property (nonatomic, strong) void (^cloundPushMessage)(NSDictionary*);

//获取ITASDK共享实例
+ (ITASDK *)sharedInstance;

/**
 启动服务
 
 @param sdk_key 极动云SDK_KEY
 @param success 成功
 @param failure NSError 具体详见文件上方NSError描述 15-19
 */
-(void)startService:(NSString*)sdk_key
            success:(void (^)(void))success
            failure:(void (^)(NSError* error))failure;

//2.停止后台服务
-(void)stopService;

//工具方法
//调用此方法检查大循环网络是否可用 Returns:false:表示大循环不可用 true:表示大循环可用
-(Boolean) isMacroCloudAvailable;

//调用此方法检查当前用户是否登录 Returns:false:表示当前用户未登录 true:表示当前用户已登录
-(Boolean) isUserLogin;


//以下为平台服务的所有功能接口，返回值（int）默认为改条接口的流水号 在回调内部也会返回该条回调的流水号 两者保持一致 用与收发匹配
// serialNumber 流水号 与调用方法的（int）一致

/** 3.注册
 *  调用此方法向平台进行用户注册
 *
 *  @param phoneNum     注册的手机号
 *  @param password     注册的用户密码，密码的长度不超过20
 *  @param verifyCode   手机验证码
 *  @param userZone     注册手机用户区号  不填 默认86
 *  @param success      返回值response字典类型  key:phoneNum
 *  @param failure      NSError 具体详见文件上方NSError描述 15-19
 *
 *  @return 流水号
 */
-(int)itaRegister:(NSString*)phoneNum
         password:(NSString*)password
       verifyCode:(NSString*)verifyCode
         userZone:(NSString*)userZone
          success:(void (^)(NSDictionary *response))success
          failure:(void (^)(NSError* error))failure;

/** 4.登录
 *  调用此方法向平台进行登录
 *
 *  @param phoneNum     在平台注册过的手机号
 *  @param password     在平台注册过的手机号对应的密码
 *  @param token        手机推送token
 *  @param userZone     注册手机用户区号  不填 默认86
 *  @param success      返回值response字典类型  key:user_id
 *  @param failure      NSError 具体详见文件上方NSError描述 15-19
 *
 *  @return 流水号
 */
-(int)itaLogin:(NSString*)phoneNum
      password:(NSString*)password
         token:(NSData*)token
      userZone:(NSString*)userZone
       success:(void (^)(NSDictionary *response))success
       failure:(void (^)(NSError* error))failure;


/* 5.登出
 调用此方法从平台登出
 */
-(void)itaLogout;

/** 6.找回密码
 *  当用户忘记密码，需找回密码时调用此方法
 *
 *  @param phoneNum     用户手机号
 *  @param verifyCode   手机验证码
 *  @param userZone     注册手机用户区号  不填 默认86
 *  @param success      返回值response字典类型  key:phoneNum
 *  @param failure      NSError 具体详见文件上方NSError描述 15-19
 *
 *  @return 流水号
 */
-(int)itaPasswordRetake:(NSString*)phoneNum
             verifyCode:(NSString*)verifyCode
            newPassword:(NSString*)password
               userZone:(NSString*)userZone
                success:(void (^)(NSDictionary *response))success
                failure:(void (^)(NSError* error))failure;


/** 7. 修改密码
 *  调用此方法修改当前登录用户密码功能
 *
 *  @param oldPassword 老密码
 *  @param newPassword 新密码
 *  @param success     返回值response字典类型  key:newPassword
 *  @param failure     NSError 具体详见文件上方NSError描述 15-19
 *
 *  @return 流水号     0106
 */
-(int)itaPasswordModifyWithOldPassword:(NSString*)oldPassword
                           newPassword:(NSString*)newPassword
                               success:(void (^)(NSDictionary *response))success
                               failure:(void (^)(NSError* error))failure;


/** 8. 添加设备
 *  调用此方法用户实现修改添加设备
 *
 *  @param deviceID     deviceID    设备ID
 *  @param success      success     返回值response字典类型  key:device_id
 *  @param failure      NSError     具体详见文件上方NSError描述 15-19
 *
 *  @return 流水号
 */
-(int)itaAddDevice:(NSString*)deviceID
           success:(void (^)(NSDictionary *response))success
           failure:(void (^)(NSError* error))failure;

/**
 *  9.删除设备
 *
 *  @param deviceID deviceID 设备ID
 *  @param success  返回值response字典类型  key:serialNumber 流水号
 *  @param failure  NSError     具体详见文件上方NSError描述 15-19
 *
 *  @return 流水号
 */
-(int)itaRemoveDevice:(NSString*)deviceID
              success:(void (^)(NSDictionary *response))success
              failure:(void (^)(NSError* error))failure;

/** 10. 修改、控制设备
 *   调用此方法 修改、控制设备
 *
 *  @param deviceID           设备ID
 *  @param propertyDictionary 设备属性字典
 *  @param success            返回值response字典类型  key:serialNumber 流水号
 *  @param failure            NSError 具体详见文件上方NSError描述 15-19
 *
 *  @return 流水号
 */
-(int)itaUpdateDevice:(NSString*)deviceID
   propertyDictionary:(NSDictionary*)propertyDictionary
              success:(void (^)(NSDictionary *response))success
              failure:(void (^)(NSError* error))failure;

/** 11. 获取设备列表
 *  获取当前登录用户的设备列表
 *  @param success              返回值response字典类型  key:deviceList  -> NSString 设备的Json字符串
 *  @param failure              NSError 具体详见文件上方NSError描述 15-19
 *
 *  @return 流水号
 */
-(int)itaGetDeviceListSuccess:(void (^)(NSDictionary *response))success
                      failure:(void (^)(NSError* error))failure;

/** 12 查询设备信息
 *  调用此方法查询设备的信息
 *
 *  @param deviceID      设备ID
 *  @param propertyArray 表示的是需要查询设备属性的字段数组 格式为@[@"",@"",@"",......]
 *  @param success       返回值response字典类型  key:propertyArray里面的Key
 *  @param failure       NSError 具体详见文件上方NSError描述 15-19
 *
 *  @return 流水号
 */
-(int)itaGetDeviceProperty:(NSString*)deviceID
             propertyArray:(NSArray*)propertyArray
                   success:(void (^)(NSDictionary *response))success
                   failure:(void (^)(NSError* error))failure;


/** 13.
 *  查找用户信息
 *
 *  @param phoneNum             手机号
 *  @param userZone             注册手机用户区号  不填 默认86
 *  @param userPropertyArray    用户信息字段
 *  @param success  返回值response字典类型  key:propertyArray里面的user_id user_name user_config
 *  @param failure  失败信息
 *
 *  @return 流水号
 */
-(int)itaSearchUserWithPhoneNum:(NSString*)phoneNum
                       userZone:(NSString*)userZone
              userPropertyArray:(NSArray*)userPropertyArray
                        success:(void (^)(NSDictionary *response))success
                        failure:(void (^)(NSError* error))failure;


/** 14.
 *  调用此方法上送用户信息
 *
 *  @param propertyDictionary   用户属性字典
 *  @param success              返回值response字典类型  key:user_version 配置文件版本号
 *  @param failure              NSError 具体详见文件上方NSError描述 15-19
 *
 *  @return 流水号
 */
-(int)itaUpdateUserInfo:(NSDictionary*)propertyDictionary
                success:(void (^)(NSDictionary *response))success
                failure:(void (^)(NSError* error))failure;

/**
 *  15 调用此方法查询设备的历史信息
 *
 *  @param deviceID         设备ID
 *  @param propertyArray    设备属性数据
 *  @param historyID        历史消息号 0为最新一条开始
 *  @param startTime        开始时间
 *  @param endTime          结束时间
 *  @param direction        方向   1向后 0向前
 *  @param queryNum         查询数量
 *  @param sort             按时间排序  默认-1  -1为逆序 1为正序
 *  @param success          返回值response字典类型  key:historyList ->NSArray<NSDictionary<key:history_id,value,time>>
 *  @param failure          NSError 具体详见文件上方NSError描述 15-19
 *
 *  @return 流水号
 */
-(int)itaGetDeviceHistoryInfo:(NSString*)deviceID
                     property:(NSArray*)propertyArray
                    historyID:(NSString*)historyID
                    startTime:(NSDate*)startTime
                      endTime:(NSDate*)endTime
                    direction:(NSString*)direction
                     queryNum:(NSString*)queryNum
                         sort:(NSString*)sort
                      success:(void (^)(NSDictionary* response))success
                      failure:(void (^)(NSError* error))failure;


/** 16
 *  自定义TAG方法
 *
 *  @param tagID    自定义TAGID
 *  @param property 自定义信息
 *  @param success  property 包含serialNumber tagID
 *  @param failure  失败消息
 *
 *  @return 流水号
 */
-(int)itaSpecialTagMethodWithTagID:(NSString*)tagID
                          property:(NSDictionary*)property
                           success:(void (^)(NSDictionary* response))success
                           failure:(void (^)(NSError* error))failure;

/**
 17 获取二维码信息
 
 @param qrCode 二维码信息
 @param success 返回值response字典类型  key:device_release、device_brand、device_iot_type、device_id、device_connect、device_key、device_open、device_mac、bluetooth_key
 @param failure NSError     具体详见文件上方NSError描述 15-19
 @return 流水号
 */
-(int)itaGetMessageWithQRCode:(NSString*)qrCode
                      success:(void (^)(NSDictionary *response))success
                      failure:(void (^)(NSError* error))failure;

/** 18. 蓝牙上送设备数据
 *   调用此方法 修改、控制设备
 *
 *  @param deviceID           设备ID
 *  @param propertyArray      设备属性数组
 *  @param success            返回值response字典类型  key:serialNumber 流水号
 *  @param failure            NSError 具体详见文件上方NSError描述 15-19
 *
 *  @return 流水号
 */
- (int)itaBluetoothUpdateDevice:(NSString*)deviceID
                   withProperty:(NSArray *)propertyArray
                        success:(void (^)(NSDictionary *response))success
                        failure:(void (^)(NSError* error))failure;


/** 19.手机号是否已经注册
 
 @param phoneNum 手机号
 @param userZone 注册手机用户区号  不填 默认86
 @param success  返回值response字典类型  key:register_status 0未注册 1已注册
 @param failure  NSError     具体详见文件上方NSError描述 15-19
 
 @return 流水
 */
- (int)itaCheckPhoneIsRegister:(NSString*)phoneNum
                      userZone:(NSString*)userZone
                       success:(void (^)(NSDictionary *response))success
                       failure:(void (^)(NSError* error))failure;




/**
 20.设置历史附属信息
 
 @param deviceID    设备ID
 @param historyID   历史信息ID 不填写history_id，平台会插入一条新的history_id，这条历史消息将没有设备数据，只有附属信息，并给一个默认的id：attachment
 @param attachment  附属信息
 @param success     返回值response字典类型  key:serialNumber 流水号
 @param failure     NSError 具体详见文件上方NSError描述 15-19
 @return 流水号
 */
-(int)itaSetHistoryAttachment:(NSString*)deviceID
                    historyID:(NSString*)historyID
                   attachment:(NSString*)attachment
                      success:(void (^)(NSDictionary* response))success
                      failure:(void (^)(NSError* error))failure;


/** 20.查询蓝牙设备注册信息

 @param deviceMac 设备Mac地址
 @param success 返回值response字典类型  key:device_id
 @param failure NSError 具体详见文件上方NSError描述 15-19
 @return 流水号
 */
-(int)itaGetBlueToothRegisterInfo:(NSString*)deviceMac
                        deviceKey:(NSString*)deviceKey
                          success:(void (^)(NSDictionary* response))success
                          failure:(void (^)(NSError* error))failure;

/**21 上送App意见
 
 
 @param advice 意见
 @param qq QQ号码
 @param success 返回值response字典类型  key:serialNumber 流水号
 @param failure NSError 具体详见文件上方NSError描述 15-19
 @return 流水号
 */
-(int)itaUploadAppAdviceCommand:(NSString*)advice
                             qq:(NSString*)qq
                        success:(void (^)(NSDictionary *response))success
                        failure:(void (^)(NSError* error))failure;

/**22 故障报修
 
 @param device_id 设备ID
 @param fault_detail 故障内容
 @param connect_way 联系方式
 @param success 返回值response字典类型  key:serialNumber 流水号
 @param failure NSError 具体详见文件上方NSError描述 15-19
 @return 流水号
 */
-(int)itaUploadDeviceFaultDetailCommand:(NSString*)device_id
                           fault_detail:(NSString*)fault_detail
                            connect_way:(NSString*)connect_way
                                success:(void (^)(NSDictionary *response))success
                                failure:(void (^)(NSError* error))failure;


/**23 是否订阅设备状态信息

 @param device_id 设备ID
 @param isSubscribe 是否订阅 0 否 1是
 @param success 返回值response字典类型 key:serialNumber 流水号
 @param failure NSError 具体详见文件上方NSError描述 15-19
 @return 流水号
 */
-(int)itaSetDevicePushMessageCommand:(NSString*)device_id
                         isSubscribe:(NSString*)isSubscribe
                             success:(void (^)(NSDictionary *response))success
                             failure:(void (^)(NSError* error))failure;

//————————————————————————————添加设备方式选择——————————————————————————————————
/** 2.1
 *  EasyLink 添加设备
 *  30秒失效
 *  @param password     wifi密码
 *  @param success      成功回调deviceID
 */
-(void)itaEasylinkStart:(NSString*)password
                success:(void (^)(NSString* deviceID))success
                failure:(void (^)(NSError* error))failure;

/** 2.2
 *  ESP 添加设备
 *  30秒失效
 *  @param password     wifi密码
 *  @param success      成功回调deviceID
 */
-(void)itaEspStart:(NSString*)password
           success:(void (^)(NSString* deviceID))success
           failure:(void (^)(NSError* error))failure;



////————————————————————————————MOB短信验证——————————————————————————————————
///** 3.1 新版本mob sdk 3.0中移除注册方法 MOBAppSecret MOBAppKey只需写入info.plist中即可
// *  MOB注册
// *
// *  @param appKey App Key
// *  @param secret Secret
// */
//-(void)mobRegisterApp:(NSString*)appKey withSecret:(NSString*)secret;


/** 3.2
 *  发送验证码
 *
 *  @param phoneNum 手机号
 *  @param success  成功返回信息
 *  @param failure  失败返回信息
 */
-(void)mobGetVerificationCode:(NSString*)phoneNum
                      success:(void (^)(void))success
                      failure:(void (^)(NSString* errorMsg))failure;




//————————————————————————————UDP——————————————————————————————————
/* 5.1
 * 调用此方法获取待添加设备信息 :小循环使用
 *  @param success       deviceID设备ID
 *  @param failure       NSError     具体详见文件上方NSError描述 15-19
 */
-(void)itaGetDeviceRegisterInfoWithDeviceKey:(NSString*)deviceKey
                                     Success:(void (^)(NSDictionary *response))success
                                     failure:(void (^)(NSError* error))failure;

//————————————————————————————蓝牙相关——————————————————————————————————

/**
 开启蓝牙服务

 @param success 成功
 @param failure 失败
 */
-(void)itaBLE_StartSuccess:(void (^)(void))success
                   failure:(void (^)(NSError* error))failure;


/**
 搜索设备
 //搜索设备 默认30秒后关闭搜索
 @param success  成功
 @param failure 失败
 */
-(void)itaBLE_SearchDevice:(void (^)(NSString*))success
                   failure:(void (^)(NSError* error))failure;


/**
 蓝牙停止搜索
 */
-(void)itaBLE_StopSearchDevice;


/**
 链接蓝牙设备

 @param mac 设备mac地址
 @param success 成功
 @param failure 失败
 */
-(void)itaBLE_ConnectDeviceWithMAC:(NSString*)mac
                           success:(void (^)(void))success
                           failure:(void (^)(NSError* error))failure;

/**
  蓝牙发送数据

 @param mac 设备mac地址
 @param data 蓝牙数据包
 @param success 成功
 @param failure 失败
 */
-(void)itaBLE_SendDataWithMAC:(NSString*)mac
                         data:(NSString*)data
                      success:(void (^)(void))success
                      failure:(void (^)(NSError* error))failure;

/**
 蓝牙发送数据

 @param mac 设备mac地址
 @param dataArray 数据数组 ：[{key:String,value:String/int,length:int},{key:String,value:String,length:int}]
 @param success 成功
 @param failure 失败
 */
-(void)itaBLE_SendDataWithMAC:(NSString*)mac
                      NSArray:(NSArray*)dataArray
                      success:(void (^)(void))success
                      failure:(void (^)(NSError* error))failure;


/**
 关闭蓝牙连接

 @param mac 设备mac地址
 */
-(void)itaBLE_Close:(NSString*)mac;


/**
 蓝牙回调

 @param callback 回调参数
 */
-(void)itaBLE_CallBack:(void (^)(int,NSString*))callback;



/**
 查询数据

 @param mac mac地址
 @param dataArray key数组
 @param success key value字典
 @param failure 失败
 */
-(void)itaBLE_QueryDataWithMAC:(NSString*)mac
                          data:(NSArray*)dataArray
                       success:(void (^)(NSDictionary*response))success
                       failure:(void (^)(NSError* error))failure;

@end
